//Arduino Eyesight Guardian by Daniel Hingston, February 2021, published on Instructables.
//This program is an adaptation of the Adafruit example sketch:
//File/examples/Adafruit ImageReader Library/BreakoutST7789 - 320x240
//See https://learn.adafruit.com/adafruit-1-14-240x135-color-tft-breakout/overview for details.

// As written, this uses the microcontroller's SPI interface for the screen
// (not 'bitbang') and must be wired to specific pins (e.g. for Arduino Uno,
// MOSI = pin 11, MISO = 12, SCK = 13). Other pins are configurable below.

#include <Adafruit_GFX.h>         // Core graphics library
#include <Adafruit_ST7789.h> // Hardware-specific library for ST7789
#include <SdFat.h>                // SD card & FAT filesystem library
#include <Adafruit_SPIFlash.h>    // SPI / QSPI flash library
#include <Adafruit_ImageReader.h> // Image-reading functions

// Comment out the next line to load from SPI/QSPI flash instead of SD card:
#define USE_SD_CARD

// TFT display and SD card share the hardware SPI interface, using
// 'select' pins for each to identify the active device on the bus.

#define SD_CS    7 // SD card select pin
#define TFT_CS  10 // TFT select pin
#define TFT_DC   8 // TFT display/command pin
#define TFT_RST  9 // Or set to -1 and connect to Arduino RESET pin

#if defined(USE_SD_CARD)
SdFat                SD;         // SD card filesystem
Adafruit_ImageReader reader(SD); // Image-reader object, pass in SD filesys
#else
// SPI or QSPI flash filesystem (i.e. CIRCUITPY drive)
#if defined(__SAMD51__) || defined(NRF52840_XXAA)
Adafruit_FlashTransport_QSPI flashTransport(PIN_QSPI_SCK, PIN_QSPI_CS,
    PIN_QSPI_IO0, PIN_QSPI_IO1, PIN_QSPI_IO2, PIN_QSPI_IO3);
#else
#if (SPI_INTERFACES_COUNT == 1)
Adafruit_FlashTransport_SPI flashTransport(SS, &SPI);
#else
Adafruit_FlashTransport_SPI flashTransport(SS1, &SPI1);
#endif
#endif
Adafruit_SPIFlash    flash(&flashTransport);
FatFileSystem        filesys;
Adafruit_ImageReader reader(filesys); // Image-reader, pass in flash filesys
#endif

Adafruit_ST7789      tft    = Adafruit_ST7789(TFT_CS, TFT_DC, TFT_RST);
Adafruit_Image       img;        // An image loaded into RAM
int32_t              width  = 0, // BMP image dimensions
                     height = 0;

//Here define which pin your push button is attached to.
const int buttonPin = 5;
int buttonState = 0;
int toggle = 0;

void setup(void) {

  ImageReturnCode stat; // Status from image-reading functions

  Serial.begin(9600);
  pinMode(buttonPin, INPUT);
  digitalWrite(buttonPin, HIGH);

#if !defined(ESP32)
  while (!Serial);      // Wait for Serial Monitor before continuing
#endif

  tft.init(135, 240);           // Init ST7789 320x240

  // The Adafruit_ImageReader constructor call (above, before setup())
  // accepts an uninitialized SdFat or FatFileSystem object. This MUST
  // BE INITIALIZED before using any of the image reader functions!
  Serial.print(F("Initializing filesystem..."));
#if defined(USE_SD_CARD)
  // SD card is pretty straightforward, a single call...
  if (!SD.begin(SD_CS, SD_SCK_MHZ(10))) { // Breakouts require 10 MHz limit due to longer wires
    Serial.println(F("SD begin() failed"));
    for (;;); // Fatal error, do not continue
  }
#else
  // SPI or QSPI flash requires two steps, one to access the bare flash
  // memory itself, then the second to access the filesystem within...
  if (!flash.begin()) {
    Serial.println(F("flash begin() failed"));
    for (;;);
  }
  if (!filesys.begin(&flash)) {
    Serial.println(F("filesys begin() failed"));
    for (;;);
  }
#endif
  Serial.println(F("OK!"));
  delay(1000); // Pause 2 seconds before moving on to loop()

  //This line draws the 'title' of the graphics, to display the title.bmp image.
  reader.drawBMP("/title.bmp", tft, 0, 0);
  //This while loop waits for the push button to be pressed before continuing.
  while (toggle == 0) {
    buttonState = digitalRead(buttonPin);
    if (buttonState == LOW) {
      toggle = 1;
    }
  }
  reader.drawBMP("/plate.bmp", tft, 0, 0);
  delay(2000);
  reader.drawBMP("/robot.bmp", tft, 0, 0);
  delay(2000);
}

void loop() {
  toggle = 0;
  //The following line makes the entire screen black.
  tft.fillScreen(0);
  //This delay command waits 20 minutes before displaying the message, you can reduce this to
  //a shorter time interval for testing!
  delay(1200000);
  reader.drawBMP("/look.bmp", tft, 0, 0);
  while (toggle == 0) {
    buttonState = digitalRead(buttonPin);
    if (buttonState == LOW) {
      toggle = 1;
    }
  }
  //The following line displays the 'well done' image after the button is pressed.
  reader.drawBMP("/well.bmp", tft, 0, 0);
  delay(2000);
}
